@@ -1,3 +1,4 @@
+from typing import Type
 from django.utils.encoding import force_text
 from jutil.dict import choices_label
 
@@ -9,7 +10,7 @@ def get_object_or_none(cls, **kwargs):
     :param kwargs: Filters for get() call
     :return: Object or None
     """
-    from django.shortcuts import _get_queryset
+    from django.shortcuts import _get_queryset  # pylint: disable=import-outside-toplevel
     qs = _get_queryset(cls)
     try:
         return qs.get(**kwargs)
@@ -19,7 +20,7 @@ def get_object_or_none(cls, **kwargs):
 
 def get_model_field_label_and_value(instance, field_name) -> (str, str):
     """
-    Returns model field label and value.
+    Returns model field label and value (as text).
     :param instance: Model instance
     :param field_name: Model attribute name
     :return: (label, value) tuple
@@ -29,13 +30,29 @@ def get_model_field_label_and_value(instance, field_name) -> (str, str):
     for f in instance._meta.fields:
         if f.attname == field_name:
             label = f.verbose_name
-            if hasattr(f, 'choices') and len(f.choices) > 0:
+            if hasattr(f, 'choices') and f.choices:
                 value = choices_label(f.choices, value)
             break
     return label, force_text(value)
 
 
-def clone_model(instance, cls, commit: bool=True, exclude_fields: tuple=('id',), base_class_suffix: str='_ptr', **kw):
+def is_model_field_changed(instance, field_name) -> bool:
+    """
+    Compares model instance field value to value stored in DB.
+    If object has not been stored in DB (yet) field is considered unchanged.
+    :param instance: Model instance
+    :param field_name: Model attribute name
+    :return: True if field value has been changed compared to value stored in DB.
+    """
+    assert hasattr(instance, field_name)
+    if not hasattr(instance, 'pk') or instance.pk is None:
+        return False
+    qs = instance.__class__.objects.all()
+    params = {'pk': instance.pk, field_name: getattr(instance, field_name)}
+    return qs.filter(**params).first() is None
+
+
+def clone_model(instance, cls: Type or None = None, commit: bool = True, exclude_fields: tuple = ('id',), base_class_suffix: str = '_ptr', **kw):
     """
     Assigns model fields to new object. Ignores exclude_fields list and
     attributes ending with pointer suffix (default '_ptr')
@@ -46,6 +63,8 @@ def clone_model(instance, cls, commit: bool=True, exclude_fields: tuple=('id',),
     :param base_class_suffix: End of name for base class pointers, e.g. model Car(Vehicle) has implicit vehicle_ptr
     :return: New instance
     """
+    if cls is None:
+        cls = instance.__class__
     keys = [f.name for f in cls._meta.fields if f.name not in exclude_fields and not f.name.endswith(base_class_suffix)]
     new_instance = cls()
     for k in keys:
